# -*- coding: utf-8 -*-
"""
Hilltop read functions.
Hilltop uses a fixed base date as 1940-01-01, while the standard unix/POSIT base date is 1970-01-01.
"""
import Hilltop
from pandas import concat, to_datetime, DataFrame

#####################################################
#### New method - not ready yet...


def ht_sites(hts, sites=None):
    """
    Function to read all of the sites in an hts file and the associated site info.

    hts -- Path to hts file (str).\n
    sites -- Optional list, array, series of site names to return.
    """

    dfile1 = Hilltop.Connect(hts)
    site_list = Hilltop.SiteList(dfile1)

    if not site_list:
        print('No sites in ' + hts)
        return(DataFrame())

    if isinstance(sites, list):
        site_list = [i for i in site_list if i in sites]

    site_info = DataFrame()

    for i in site_list:
        try:
            info1 = Hilltop.MeasurementList(dfile1, i)
        except SystemError as err:
            print('Site ' + str(i) + " didn't work. Error: " + str(err))
            continue
        info1.loc[:, 'site'] = i.encode('ascii', 'ignore').decode()
        site_info = concat([site_info, info1])
    site_info.reset_index(drop=True, inplace=True)

    site_info.loc[:, 'Start Time'] = to_datetime(site_info.loc[:, 'Start Time'], format='%d-%b-%Y %H:%M:%S')
    site_info.loc[:, 'End Time'] = to_datetime(site_info.loc[:, 'End Time'], format='%d-%b-%Y %H:%M:%S')

    len_all = len(site_list)
    len_got = len(site_info.site.unique())
    print('Missing ' + str(len_all - len_got) + ' sites, which is ' + str(round(100 * ((len_all - len_got)/len_all), 1)) + '% of the total')

    Hilltop.Disconnect(dfile1)
    return site_info


def ht_get_data(hts, sites=None, from_date=None, to_date=None, agg_method='Average', interval='1 day', alignment='00:00', output_missing_sites=False):
    """

    """

    site_info = ht_sites(hts, sites=sites)

    if isinstance(from_date, str):
        from_date1 = to_datetime(from_date).strftime('%d-%b-%Y %H:%M')
    else:
        from_date1 = ''
    if isinstance(to_date, str):
        to_date1 = to_datetime(to_date).strftime('%d-%b-%Y %H:%M')
    else:
        to_date1 = ''

    data1 = []
    missing_site_data = DataFrame()

    dfile1 = Hilltop.Connect(hts)
    for i in site_info.index:
        site = site_info.loc[i, 'site']
        mtype = site_info.loc[i, 'Measurement']

        d1 = Hilltop.GetData(dfile1, site, mtype, from_date1, to_date1, method=agg_method, interval=interval, alignment=alignment)

        if d1.empty:
            missing_site_data = concat([missing_site_data, site_info.loc[i]])
            print('No data for site ' + str(site) + ' and mtype ' + str(mtype))
            continue

        if (interval == '1 day') & (alignment == '00:00'):
            d1.index = d1.index.normalize()
        d1.name = 'data'
        d1.index.name = 'time'
        d2 = d1.reset_index()
        d2.loc[:, 'site'] = site
        d2.loc[:, 'mtype'] = mtype
        print(site, mtype)

        data1.append(d2)

    try:
        data2 = concat(data1)
    except MemoryError:
        print('Not enough memory for a 32bit application')

    if missing_site_data.empty:
        print('No missing data for any sites/mtypes')
    else:
        print('No data for ' + str(len(missing_site_data)) + ' sites/mtype combos')
        if output_missing_sites:
            return(data2, missing_site_data)
    return data2




# -*- coding: utf-8 -*-
"""
Utility functions for Hilltop functions.
"""
from os import path
from datetime import datetime
try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import SafeConfigParser as ConfigParser


def parse_dsn(dsn_path):
    """
    Function to parse a dsn file and all sub-dsn files into paths to hts files. Returns a list of hts paths.

    Parameters
    ----------
    dsn_path : str
        Path to the dsn file.

    Returns
    -------
    List of path strings to hts files.
    """

    base_path = path.dirname(dsn_path)

    dsn = ConfigParser()
    dsn.read(dsn_path)
    files1 = [os.path.join(base_path, i[1]) for i in dsn.items('Hilltop') if 'file' in i[0]]
    hts1 = [i for i in files1 if i.endswith('.hts')]
    dsn1 = [i for i in files1 if i.endswith('.dsn')]
    while dsn1:
        for f in dsn1:
            base_path = path.dirname(f)
            p1 = ConfigParser()
            p1.read(f)
            files1 = [path.join(base_path, i[1]) for i in p1.items('Hilltop') if 'file' in i[0]]
            hts1.extend([i for i in files1 if i.endswith('.hts')])
            dsn1.remove(f)
            dsn1[0:0] = [i for i in files1 if i.endswith('.dsn')]
    return hts1


def pytime_to_datetime(pytime):
    """
    Function to convert a PyTime object to a datetime object.
    """

    dt1 = datetime(year=pytime.year, month=pytime.month, day=pytime.day, hour=pytime.hour, minute=pytime.minute)
    return dt1


def time_switch(x):
    """
    Convenience codes to convert for time text to pandas time codes.
    """
    return {
        'min': 'Min',
        'mins': 'Min',
        'minute': 'Min',
        'minutes': 'Min',
        'hour': 'H',
        'hours': 'H',
        'day': 'D',
        'days': 'D',
        'week': 'W',
        'weeks': 'W',
        'month': 'M',
        'months': 'M',
        'year': 'A',
        'years': 'A',
        'water year': 'A-JUN',
        'water years': 'A-JUN',
    }.get(x, 'A')
